# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdlib.h>
   # include "mycurses.h"
   # include "message.h"
   # include "cwrite.h"
   # include "command.h"
   # include "command_line.h"
   # include "key.h"
   # include "cursor.h"
   # include "history.h"
   # include "string.h"
# endif
# if GLOBAL
   char * _command_line;
   char * _user_line;
   int command_line_stop = 0;
   int command_line_size;
   static int cursor_y, cursor_x;

   char * _confirm_str[] = 
      { "Yes",
        "No",
      };
   char * _search_rstr[] = 
      { "Yes",
        "No",
        "All",
      };
   char * _set_str[] =
      { "color theme",
        "syntax",
        "tab",
      };
   char * _color_theme_str[] =
      { "Standard",
        "New",
        "Midnight",
        "Blue"
      };
   char * _syntax_str[] =
      { "None",
        "C/C++",
      };
   int set_cardinal = sizeof(_set_str)/sizeof(char *);
   int syntax_cardinal = sizeof(_syntax_str)/sizeof(char *);
# endif
# if PROTO
   void command_line_read(History * _history);
   void command_line_redraw();
# endif

void command_line_init()
   { _command_line = (char *) malloc(screen_cols);
     _memset(_command_line, ' ', screen_cols);
     _user_line = (char *) malloc(screen_cols);
      return;
   }
void command_line_copy(char * _str)
   { string_copy(_command_line + command_line_stop, _str);
     command_line_size = string_len(_str);
     //command_line_stop = command_line_size;
     return;
   }
void command_line_add(char *_str)
   { string_copy(_command_line + command_line_stop, _str);
     int len = string_len(_str);
     command_line_size += len;
     cursor_x += len;
     return;
   }
void command_line_print(char * _str)
   { command_line_copy(_str);
     //command_line_stop = string_len(_str);
     command_line_redraw();
     return;
   }
void command_line_print_stop(char * _str)
   { command_line_copy(_str);
     command_line_stop = string_len(_str);
     command_line_redraw();

     return;
   }
void command_line_echo(char * _str)
   { command_line_copy(_str);
     command_line_stop = string_len(_str);
     command_line_redraw();
     return;
   }
void command_line_vprintf(char * _template, ...)
   { va_list ap;
     va_start(ap, _template);
     vsprintf(_command_line, _template, ap);
     return;
   }
void command_line_redraw()
   { curses_mvaddnstr(COMMAND_LINE, 0, _command_line, screen_cols);
     cursor_move(cursor_y, cursor_x);

     return;
   }
void command_line_redraw2()
   { curses_mvaddnstr(COMMAND_LINE, 0, _command_line, screen_cols);
     cursor_move(cursor_y, command_line_size);

     return;
   }
void command_line_clear()
   { _memset(_command_line, ' ', screen_cols);
     command_line_stop = 0;

     return;
   }
void command_line_back_to_stop()
   { char * _command_line_point = _command_line + command_line_stop;
     int cols = screen_cols - command_line_stop;

     for ( int i = 0; i < cols; i++ )
        { * _command_line_point++ = ' ';
        }
     cursor_x = command_line_stop;
     command_line_redraw();
     cursor_move(cursor_y, cursor_x);

     return;
   }
void command_line_backspace()
   { if ( command_line_size == command_line_stop )
        { return;
        }
     * (_command_line + --command_line_size) = ' ';
     cursor_x--;

     return;
   }
void command_line_enter()
   { int message_len = string_len(_message[3]);
     char * _command_line_point = _command_line + message_len;
     char * _user_line_point = _user_line;
     command_line_stop = message_len;
     command_line_size = message_len;

     string_copy(_command_line, _message[3]);

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;

     command_line_read(_history[HISTORY_COMMAND]);
     if ( * _user_line )
        { int n = command_recognize();
          if ( n >= 0 )
             { command_execute(n);
             }
        }
     command_line_clear();
     command_line_redraw();
     cursor_move(* _cursor_y, * _cursor_x);
     return;
}
void command_line_read()
   { char * _command_line_point = _command_line + command_line_stop;
     char * _user_line_point = _user_line;

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;
      command_line_size = command_line_stop;
     int c;

     do
        { command_line_redraw();
          c = curses_getch();
          if ( c == KEY_BACKSPACE || c == CTRL_H )
             { if ( command_line_size == command_line_stop )
                  { continue;
                  }
                command_line_backspace();
               _command_line_point--;
               _user_line_point--;
               continue;
             }
          if ( c > 27 && c < 256 )
             { //* _command_line_point++ = c;
               * _command_line_point++ = '*';
               command_line_size++;
               * _user_line_point++ = c;
               cursor_x++;
             }
        }
     while ( c && c != CTRL_J && c != CTRL_M );

     if ( ! c )
        { _command_line_point = _command_line;
          _user_line_point = _user_line;
          * _user_line_point = 0;
        }
     else
        { * _command_line_point = 0;
          * _user_line_point = 0;
          //string_copy(_user_line, _command_line + command_line_stop);
        }
 
     command_line_clear();
     command_line_redraw();

     cursor_move(* _cursor_y, * _cursor_x);
     return;
   }
void command_line_read_number()
   { char * _command_line_point = _command_line + command_line_stop;
     char * _user_line_point = _user_line;

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;
      command_line_size = command_line_stop;
     int c;

     do
        { command_line_redraw();
          c = curses_getch();
          if ( c == KEY_BACKSPACE || c == CTRL_H )
             { if ( command_line_size == command_line_stop )
                  { continue;
                  }
                command_line_backspace();
               _command_line_point--;
               _user_line_point--;
               continue;
             }
          //if ( c > 27 && c < 256 )
          if ( c >= '0' && c <= '9' )
             { * _command_line_point++ = c;
               //* _command_line_point++ = '*';
               command_line_size++;
               * _user_line_point++ = c;
               cursor_x++;
             }
        }
     while ( c && c != CTRL_J && c != CTRL_M );

     if ( ! c )
        { _command_line_point = _command_line;
          _user_line_point = _user_line;
          * _user_line_point = 0;
        }
     else
        { * _command_line_point = 0;
          * _user_line_point = 0;
          //string_copy(_user_line, _command_line + command_line_stop);
        }
 
     command_line_clear();
     command_line_redraw();

     cursor_move(* _cursor_y, * _cursor_x);
     return;
   }
void command_line_read(History * _history)
   { char * _command_line_point = _command_line + command_line_stop;
     char * _user_line_point = _user_line;

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;
      command_line_size = command_line_stop;
     int c;
     bool history_match = 0;

     do
        { command_line_redraw();
          c = curses_getch();
          if ( c == KEY_UP || c == CTRL_P )
             { * _user_line_point = 0;
               c = history_complete(_history);
               if ( c > 1 )
                  { string_copy(_user_line, _command_line + command_line_stop);
                    _command_line_point = _command_line + command_line_size;
                  }
               if ( c == 1 )
                  { cursor_x = command_line_size;
                    command_line_redraw();
                  }
             }
          if ( c == KEY_BACKSPACE || c == CTRL_H )
             { if ( command_line_size == command_line_stop )
                  { continue;
                  }
                command_line_backspace();
               _command_line_point--;
               _user_line_point--;
               continue;
             }
          if ( c > 27 && c < 256 )
             { * _command_line_point++ = c;
               command_line_size++;
               * _user_line_point++ = c;
               cursor_x++;
             }
        }
     while ( c && c != CTRL_J && c != CTRL_M );

     if ( ! c )
        { _command_line_point = _command_line;
          _user_line_point = _user_line;
          * _user_line_point = 0;
        }
     else
        { * _command_line_point = 0;
          string_copy(_user_line, _command_line + command_line_stop);
          history_update(_history);
        }
     //if ( ! history_match )
     //   { * _command_line_point = 0;
     //     * _user_line_point = 0;
     //   }
     
     command_line_clear();
     command_line_redraw();

     cursor_move(* _cursor_y, * _cursor_x);
     return;
   }
int  command_line_revolve(char * _revolve_str[], int start, int n)
   { cursor_hide();

     int k = start;
     int c;
     command_line_stop = command_line_size;
     command_line_print(_revolve_str[start]);
     do
       { c = curses_getch();
         switch (c)
            { case KEY_UP:
              case CTRL_P:
                 if ( k )
                    { k--;
                      command_line_back_to_stop();
                      command_line_print(_revolve_str[k]);
                    }
                 break;
              case KEY_DOWN:
              case CTRL_N:
                 if ( k < n - 1 )
                    { k++;
                      command_line_back_to_stop();
                      command_line_print(_revolve_str[k]);
                    }
                 break;
              case CTRL_M:
              case CTRL_J:
                 //cursor_show();
                 //return k;
                 c = 0;
                 break;
              case 0:
                 k = REPLY_CANCEL;
                 break;
            }
       }
    while ( c );

    cursor_show();
    command_line_clear();
    command_line_redraw();

    return k;
   }
