# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdio.h>
   # include <stdlib.h>
   # include <curses.h>
   # include <unistd.h>
   # include <sys/types.h>
   # include <pwd.h>
   # include <sys/stat.h>
   # include <errno.h>
   # include "mycurses.h"
   # include "cwrite.h"
   # include "command.h"
   # include "command_line.h"
   # include "header_line.h"
   # include "status_line.h"
   # include "message.h"
   # include "key.h"
   # include "datasystem.h"
   # include "screen.h"
   # include "cursor.h"
   # include "search.h"
   # include "history.h"
   # include "preferences.h"
   # include "bookmark.h"
   # include "string.h"
# endif
# if GLOBAL
   passwd * _user_data;
   spwd * _user_spwd;
   char * _user_dir;
   char * _user_history;
   char * _user_preferences;
   char * _user_selection;

   short header_color_pair = 1;
   short status_color_pair = 2;
   short screen_color_pair = 3;
   short block_color_pair = 4;
   short selection_color_pair = 5;
   short search_color_pair = 6;
   short browse_color_pair = 7;

   bool exit_request = 0;

   char * _help_filename = "/usr/share/cwrite/help.txt";
# endif
# if TEST
   # define TEST_USER_INFO
# endif

void get_user_info()
   {    // -- get user data --
     _user_data = (passwd *) calloc(1, sizeof(passwd));
     uid_t user_id = getuid();
     passwd * _user_data_tmp = getpwuid(user_id);
     memcopy((char *)_user_data, (char *)_user_data_tmp, sizeof(passwd));
     # ifdef TEST_USER_INFO
        printf("_user_data->pwdir = %s\n", _user_data->pw_dir);
     # endif
     _user_dir = (char *) calloc(1, 256);
     string_copy(_user_dir, _user_data->pw_dir);
     string_append(_user_dir, "/.cwrite/");
     # ifdef TEST_USER_INFO
        printf("_user_dir = %s\n", _user_dir);
        printf("user_id = %d\n", user_id);
        printf("user_euid = %d\n", geteuid());
     # endif

        // -- get shadow password --
     # ifdef LOCK_OPTION
       _user_spwd = (spwd *) calloc(1, sizeof(spwd));
       spwd * _user_spwd_tmp = getspnam(_user_data->pw_name);
       memcopy((char *)_user_spwd, (char *)_user_spwd_tmp, sizeof(spwd));
     # endif
        // -- create directory --
     //mkdir(_user_dir, S_IRUSR | S_IWUSR);
     //mkdir(_user_dir, S_IRWXU | S_IRGRP | S_IWGRP);
     mkdir(_user_dir, S_IRWXU | S_IRWXG);

        // -- set _user_history --
     _user_history = (char *) calloc(1, USER_FILENAME_MAX);
     string_copy(_user_history, _user_dir);
     string_append(_user_history, "history");
     # ifdef TEST_USER_INFO
        printf("_user_dir = %s\n", _user_history);
     # endif

        // -- set _user_preferences --
     _user_preferences = (char *) calloc(1, USER_FILENAME_MAX);
     string_copy(_user_preferences, _user_dir);
     string_append(_user_preferences, "preferences");
     # ifdef TEST_USER_INFO
        printf("_user_preferences = %s\n", _user_preferences);
     # endif

        // -- set _user_selection -- 
     _user_selection = (char *) calloc(1, USER_FILENAME_MAX);
     string_copy(_user_selection, _user_dir);
     string_append(_user_selection, "selection");
     # ifdef TEST_USER_INFO
        printf("_user_selection = %s\n", _user_selection);
        //exit(0);
     # endif

     return;
   }

main(int argc, char * _argv[])
   { int c;
     printf("hello\n");
     get_user_info();
     //exit(0);
     
        // -- init screen --
     screen_init();
     screen_cursor();

        // -- load messages --
     message_load();
     //message_print(); 
     //return;

        // -- init curses --
     initscr(); cbreak(); noecho();
     nonl();
     intrflush(stdscr, FALSE);
     keypad(stdscr, TRUE);
     raw();
 
        // -- init and load history -- 
     history_init();
     history_load();

        // -- init and load preferences --
     preferences_init();
     preferences_load();

        // -- init colors --
     start_color();
     color_theme_init();
     color_theme_set(_preferences->color_theme);
      //endwin();
      //exit(0);
    
        // -- init lines --
     header_line_init();
     status_line_init();
     command_line_init();
     search_init();

        // -- draw lines --
     cursor_hide();
     header_line_greet();
     header_line_redraw();
     status_line_redraw();
     //exit(0);

        // --  alloc datasystems --
     datasystem_alloc();

     do
        {    // -- open file on start --
          if ( argc == 2 )
             { command_open(_argv[1]);
               argc = 0;
               c = 1;
             }
          else
             { c = curses_getch();
             }
          status_line_clear();
          status_line_redraw();
          cursor_show();

          switch (c)
             { case 0:
                  command_line_enter();
                  break;
               case 1:
                  break;
               case CTRL_N:
                  command_new();
                  break;
               case CTRL_O:
                  command_open(0);
                  break;
               case CTRL_Q:
                  command_exit();
                  break;
               default:
                  cursor_hide();  
                  continue;
             }
          if ( datasystem_count )
             { get_input();
               return_request = 0;
             }
          cursor_hide();
         }
     while ( ! exit_request);

     endwin();

        // -- save history and preferences -- 
     history_save();
     preferences_save();

     return 0;
   }
