# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdio.h>
   # include <stdlib.h>
   # include <fcntl.h>
   # include <unistd.h>
   # include "mycurses.h"
   # include "cwrite.h"
   # include "failure.h"
   # include "command_line.h"
   # include "key.h"
   # include "history.h"
   # include "block.h"
   # include "string.h"
# endif
# if GLOBAL
   History * _history[NUMBER_OF_HISTORIES];
# endif

void history_init()
   { for (int i = 0; i < NUMBER_OF_HISTORIES; i++)
        { _history[i] = (History *) calloc(1, sizeof(History));
          _history[i]->cardinal = 0;
             for (int j = 0; j < ENTRIES_IN_HISTORY; j++) 
                { _history[i]->_entry[j] = (char *) calloc(1, HISTORY_ENTRY_SIZE_MAX);
                }
        }
     return;
   }

void history_save()
   { int sid = 104;
     FILE * _file = fopen(_user_history, "w+");
     if ( ! _file )
        { fail(sid, "Failed to creat history file");
        }
     fclose(_file);
     int fd = open(_user_history, O_WRONLY);
     if ( fd == -1 )
        { return;
        }
     for (int i = 0; i < NUMBER_OF_HISTORIES; i++)
        { write(fd, & _history[i]->cardinal, sizeof(_history[i]->cardinal));
          for (int j = 0; j < _history[i]->cardinal; j++)
             { _history[i]->entry_size[j] = string_len(_history[i]->_entry[j]);
               write(fd, & _history[i]->entry_size[j], sizeof(_history[i]->entry_size[j]));
               write(fd, _history[i]->_entry[j], _history[i]->entry_size[j]);
             }
        }
     close(fd);
   }
void history_load()
   { int fd = open(_user_history, O_RDONLY);
     for (int i = 0; i < NUMBER_OF_HISTORIES; i++)
        { read(fd, & _history[i]->cardinal, sizeof(_history[i]->cardinal));
          for (int j = 0; j < _history[i]->cardinal; j++)
             { read(fd, & _history[i]->entry_size[j], sizeof(_history[i]->entry_size[j]));
               read(fd, _history[i]->_entry[j], _history[i]->entry_size[j]);
             }
        }
     close(fd);
   }

int  history_complete(History * _history)
   { int ordinal = _history->cardinal;
     int key = KEY_UP;
     do
        { 
          switch(key)
             { case KEY_UP:
               case CTRL_P:
                  if ( ! ordinal )
                     {    // -- walk back if no match --
                       if ( * _user_line && ! line_match(_history->_entry[ordinal], _user_line) )
                          { key = CTRL_N;
                            continue;
                          }
                       else
                          { break;
                          }
                     }
                  ordinal--;
                  if ( ! line_match(_history->_entry[ordinal], _user_line) )
                     { continue;
                     }
                  command_line_size = command_line_stop;
                   command_line_back_to_stop();
                  command_line_add(_history->_entry[ordinal]);
                  command_line_redraw2();
                  break;
               case KEY_DOWN:
               case CTRL_N:
                  if ( ordinal == _history->cardinal - 1 )
                     { command_line_back_to_stop();
                       string_copy(_command_line + command_line_stop, _user_line);
                       command_line_size = command_line_stop + string_len(_user_line);
                       //command_line_redraw();
                       return 1;
                     }
                  ordinal++;
                  if ( ! line_match(_history->_entry[ordinal], _user_line) )
                     { continue;
                     }
                  command_line_size = command_line_stop;
                   command_line_back_to_stop();
                  command_line_add(_history->_entry[ordinal]);
                  command_line_redraw2();
                  break;
               //case KEY_BACKSPACE:
               //case CTRL_H:
                  //if ( command_line_size == command_line_stop )
                  //   { break;
                  //   }
                  //command_line_backspace();
                  //command_line_redraw();
                  //break;
               default:
                  return(key);
             }
          key = curses_getch();
        }
     while ( key && key != CTRL_M && key != CTRL_J);

     return(key);
   }
void history_shift(History * _history)
   { char * _history_entry_tmp = _history->_entry[0];
     for (int i = 0; i < ENTRIES_IN_HISTORY-1; i++)
        { _history->_entry[i] = _history->_entry[i+1];
        }
     _history->_entry[ENTRIES_IN_HISTORY-1] = _history_entry_tmp;
     _history->cardinal--;
     return;
   }
void history_update(History * _history)
   { int cardinal = _history->cardinal;
     int i;
     bool history_match = 0;
     for ( i = 0; i < cardinal; i++)
        { if ( string_compare(_history->_entry[i], _user_line) )
             { history_match = 1;
               break;   
             }
        }
     int ordinal = i;
     if ( history_match ) 
        { char * _history_line_tmp = _history->_entry[ordinal];
          for (int i = ordinal; i < cardinal; i++)
             { _history->_entry[i] = _history->_entry[i+1];
             }
          _history->_entry[cardinal-1] = _history_line_tmp;
        }
     else 
        { if ( _history->cardinal == ENTRIES_IN_HISTORY )
             { history_shift(_history);
               cardinal = _history->cardinal;
             }
          string_copy(_history->_entry[cardinal], _user_line);
          _history->cardinal++;
        }
     return;
   }
